#version 150

#moj_import <light.glsl>
#moj_import <fog.glsl>

#moj_import <map_data.glsl>

in vec3 Position;
in vec4 Color;
in vec2 UV0;
in ivec2 UV2;
in vec3 Normal;

uniform sampler2D Sampler0;
uniform sampler2D Sampler2;

uniform mat4 ModelViewMat;
uniform mat4 ProjMat;
uniform vec3 ChunkOffset;
uniform int FogShape;

uniform float GameTime;

out vec3 chunkPosition;

out float vertexDistance;
out vec4 vertexColor;
out vec2 texCoord0;
out vec4 normal;

void main() {
    vec4 texCol = texture(Sampler0, UV0);

    vec3 pos = Position + ChunkOffset;

    if (texCol.a >= 0.7 && texCol.a < 0.9) {
        if (ModelViewMat[0].x > 0.0) {
            pos.z += REPEAT_LENGTH;
        }

        pos.z -= fract(GameTime * BLOCK_MOVE_SPEED) * REPEAT_LENGTH;
    }
    else if (texCol.a >= 0.5 && texCol.a < 0.7) {
        pos.y -= 8.0;
        pos += Normal * BLOCK_OVERLAY_OFFSET;
    }
    else if (texCol.a >= 0.3 && texCol.a < 0.5) {
        pos.y += 8.0;
        pos += Normal * BLOCK_OVERLAY_OFFSET;
    }

    chunkPosition = ChunkOffset;

    gl_Position = ProjMat * ModelViewMat * vec4(pos, 1.0);

    vertexDistance = fog_distance(ModelViewMat, pos, FogShape);
    vertexColor = Color * minecraft_sample_lightmap(Sampler2, UV2);
    texCoord0 = UV0;
    normal = ProjMat * ModelViewMat * vec4(Normal, 0.0);
}
